@@ -44,10 +44,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE fft_base,             ONLY : dfftp, dffts
   USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
   USE wvfct,                ONLY : nbnd, npwx
-  USE scf,                  ONLY : rho, vrs
-#if defined(__CUDA)
-  USE scf_gpum,             ONLY : vrs_d
-#endif
+  USE scf,                  ONLY : rho
   USE uspp,                 ONLY : okvan, vkb, deeq_nc
   USE uspp_param,           ONLY : nhm
   USE noncollin_module,     ONLY : noncolin, domag, npol, nspin_mag
@@ -85,12 +82,18 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE dv_of_drho_lr,        ONLY : dv_of_drho
   USE fft_interfaces,       ONLY : fft_interpolate
   USE ldaU,                 ONLY : lda_plus_u
-  USE nc_mag_aux,           ONLY : int1_nc_save, deeq_nc_save, int3_save
   USE apply_dpot_mod,       ONLY : apply_dpot_allocate, apply_dpot_deallocate
   USE response_kernels,     ONLY : sternheimer_kernel
   USE uspp_init,            ONLY : init_us_2
   USE sym_def_module,       ONLY : sym_def
-  implicit none
+  USE lr_nc_mag,            ONLY : int1_nc_save, deeq_nc_save, int3_nc_save
+  USE two_chem,             ONLY : twochem
+  !FIXME  make explicit mentions of lr_two_chem variable that are used
+  USE lr_two_chem           !ONLY : def_val, def_cond, drhoscf_cond, drhoscfh_cond,&
+                            !       dbecsum_cond,dbecsum_cond_nc,ldos_cond,ldoss_cond,&
+                            !       dos_ef_cond,becsum1_cond
+                            !twochem variables
+   implicit none
 
   integer :: irr
   !! input: the irreducible representation
@@ -188,7 +191,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   allocate (aux2(npwx*npol, nbnd))
   allocate (drhoc(dfftp%nnr))
   IF (noncolin.AND.domag.AND.okvan) THEN
-     ALLOCATE (int3_save( nhm, nhm, nat, nspin_mag, npe, 2))
+     ALLOCATE (int3_nc_save( nhm, nhm, nat, nspin_mag, npe, 2))
      ALLOCATE (dbecsum_aux ( (nhm * (nhm + 1))/2 , nat , nspin_mag , npe))
   ENDIF
   CALL apply_dpot_allocate()
@@ -232,6 +235,19 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      allocate (becsum1 ( (nhm * (nhm + 1))/2 , nat , nspin_mag))
      call localdos ( ldos , ldoss , becsum1, dos_ef )
      IF (.NOT.okpaw) deallocate(becsum1)
+     !
+     if (twochem) then
+         IF (noncolin) allocate (dbecsum_cond_nc (nhm,nhm, nat , nspin , npe, nsolv))
+         allocate (drhoscf_cond ( dfftp%nnr, nspin_mag , npe))
+         allocate (drhoscfh_cond ( dfftp%nnr, nspin_mag , npe))
+         allocate (dbecsum_cond ( (nhm * (nhm + 1))/2 , nat , nspin_mag , npe))
+         allocate ( ldos_cond( dfftp%nnr  , nspin_mag) )
+         allocate ( ldoss_cond( dffts%nnr , nspin_mag) )
+         allocate (becsum1_cond ( (nhm * (nhm + 1))/2 , nat , nspin_mag))
+         call localdos_cond ( ldos_cond , ldoss_cond , becsum1_cond, dos_ef_cond )
+         IF (.NOT.okpaw) deallocate(becsum1_cond)
+     end if
+     !twochem allocations
   endif
   !
   !
@@ -267,6 +283,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         !
         IF (nksq > 1 .OR. nsolv == 2) THEN
            CALL get_buffer(evc, lrwfc, iuwfc, ikmk)
+           !$acc update device(evc)
         ENDIF
         !
         DO ipert = 1, npe
@@ -313,6 +330,9 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      drhoscf = (0.d0, 0.d0)
      dbecsum = (0.d0, 0.d0)
      IF (noncolin) dbecsum_nc = (0.d0, 0.d0)
+     IF (lmetq0.and.twochem) drhoscf_cond = (0.d0, 0.d0)
+     IF (lmetq0.and.twochem) dbecsum_cond = (0.d0, 0.d0)
+     IF (noncolin.and.lmetq0.and.twochem) dbecsum_cond_nc = (0.d0, 0.d0)
      !
      ! DFPT+U: at each ph iteration calculate dnsscf,
      ! i.e. the scf variation of the occupation matrix ns.
@@ -322,23 +342,6 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      ! Start the loop on the two linear systems, one at B and one at -B
      !
      DO isolv = 1, nsolv
-        !
-        !  change the sign of the magnetic field if required
-        !
-        IF (isolv == 2) THEN
-           IF (.NOT. first_iter) THEN
-              dvscfins(:, 2:4, :) = -dvscfins(:, 2:4, :)
-              IF (okvan) int3_nc(:,:,:,:,:) = int3_save(:,:,:,:,:,2)
-           ENDIF
-           vrs(:, 2:4) = -vrs(:, 2:4)
-#if defined(__CUDA)
-           vrs_d = vrs
-#endif
-           IF (okvan) THEN
-                   deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,2)
-                   !$acc update device(deeq_nc)
-           ENDIF
-        ENDIF
         !
         ! set threshold for iterative solution of the linear system
         !
@@ -347,29 +350,21 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         ELSE
            thresh = min (1.d-1 * sqrt (dr2), 1.d-2)
         ENDIF
+        if (.not.(twochem.and.lmetq0)) then
         !
         ! Compute drhoscf, the charge density response to the total potential
         !
         CALL sternheimer_kernel(first_iter, isolv==2, npe, lrbar, iubar, &
             thresh, dvscfins, all_conv, averlt, drhoscf, dbecsum, &
             dbecsum_nc(:,:,:,:,:,isolv))
+        else
         !
-        !  reset the original magnetic field if it was changed
+        ! Compute drhoscf and drhoscf_cond for the twochem case
         !
-        IF (isolv == 2) THEN
-           IF (.NOT. first_iter) THEN
-              dvscfins(:, 2:4, :) = -dvscfins(:, 2:4, :)
-              IF (okvan) int3_nc(:,:,:,:,:) = int3_save(:,:,:,:,:,1)
-           ENDIF
-           vrs(:, 2:4) = -vrs(:, 2:4)
-#if defined(__CUDA)
-           vrs_d = vrs
-#endif
-           IF (okvan) THEN
-                   deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,1)
-                   !$acc update device(deeq_nc)
-           ENDIF
-        ENDIF
+        CALL sternheimer_kernel_twochem(first_iter, isolv==2, npe, lrbar, iubar, &
+            thresh, dvscfins, all_conv, averlt, drhoscf, dbecsum, &
+            dbecsum_nc(:,:,:,:,:,isolv),drhoscf_cond,dbecsum_cond,dbecsum_cond_nc)
+        end if
         !
      END DO ! isolv
      !
@@ -377,25 +372,36 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         drhoscf = drhoscf / 2.0_DP
         dbecsum = dbecsum / 2.0_DP
         dbecsum_nc = dbecsum_nc / 2.0_DP
+        !twochem at gamma 
+        IF (lmetq0.AND.twochem) THEN
+                drhoscf = drhoscf / 2.0_DP
+                dbecsum = dbecsum / 2.0_DP
+                dbecsum_nc = dbecsum_nc / 2.0_DP
+        END IF
      ENDIF
+
      !
      !  The calculation of dbecsum is distributed across processors (see addusdbec)
      !  Sum over processors the contributions coming from each slice of bands
      !
      IF (noncolin) THEN
         call mp_sum ( dbecsum_nc, intra_bgrp_comm )
+        if(twochem.and.lmetq0) call mp_sum ( dbecsum_cond_nc, intra_bgrp_comm )
      ELSE
         call mp_sum ( dbecsum, intra_bgrp_comm )
+        if(twochem.and.lmetq0) call mp_sum ( dbecsum_cond, intra_bgrp_comm )
      ENDIF
 
      if (doublegrid) then
         do is = 1, nspin_mag
            do ipert = 1, npe
               call fft_interpolate (dffts, drhoscf(:,is,ipert), dfftp, drhoscfh(:,is,ipert))
+              if (twochem.and.lmetq0) call fft_interpolate (dffts, drhoscf_cond(:,is,ipert), dfftp, drhoscfh_cond(:,is,ipert))
            enddo
         enddo
      else
         call zcopy (npe*nspin_mag*dfftp%nnr, drhoscf, 1, drhoscfh, 1)
+        if (twochem.and.lmetq0) call zcopy (npe*nspin_mag*dfftp%nnr, drhoscf_cond, 1, drhoscfh_cond, 1)
      endif
      !
      !  In the noncolinear, spin-orbit case rotate dbecsum
@@ -409,16 +415,33 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
            dbecsum(:,:,2:4,:)=dbecsum(:,:,2:4,:)-dbecsum_aux(:,:,2:4,:)
         ENDIF
      ENDIF
+     !rotate also dbecsum_cond in the twochem case
+     IF (noncolin.and.okvan.and.lmetq0.and.twochem) THEN
+        CALL set_dbecsum_nc(dbecsum_nc, dbecsum, npe)
+        IF (nsolv==2) THEN
+           dbecsum_aux=(0.0_DP,0.0_DP)
+           CALL set_dbecsum_nc(dbecsum_cond_nc(1,1,1,1,1,2), dbecsum_aux, npe)
+           dbecsum_cond(:,:,1,:)=dbecsum_cond(:,:,1,:)+dbecsum_aux(:,:,1,:)
+           dbecsum_cond(:,:,2:4,:)=dbecsum_cond(:,:,2:4,:)-dbecsum_aux(:,:,2:4,:)
+        ENDIF
+     ENDIF
+
      !
      !    Now we compute for all perturbations the total charge and potential
      !
      call addusddens (drhoscfh, dbecsum, imode0, npe, 0)
+     IF (twochem.and.lmetq0) call addusddens_cond (drhoscfh_cond, dbecsum_cond, imode0, npe, 0) !twochem case
      !
      !   Reduce the delta rho across pools
      !
      call mp_sum ( drhoscf, inter_pool_comm )
      call mp_sum ( drhoscfh, inter_pool_comm )
      IF (okpaw) call mp_sum ( dbecsum, inter_pool_comm )
+     !twochem case
+     IF (twochem.and.lmetq0) call mp_sum ( drhoscf_cond, inter_pool_comm )
+     IF (twochem.and.lmetq0) call mp_sum ( drhoscfh_cond, inter_pool_comm )
+     IF (okpaw.and.twochem.and.lmetq0) call mp_sum ( dbecsum_cond, inter_pool_comm )
+
      !
      IF (okpaw) THEN
         DO ipert=1,npe
@@ -431,9 +454,13 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      !
      IF (lmetq0) THEN
         IF (okpaw) THEN
-           CALL ef_shift(npe, dos_ef, ldos, drhoscfh, dbecsum, becsum1, irr, sym_def)
+           CALL ef_shift(npe, dos_ef, ldos, drhoscfh, dbecsum, becsum1, sym_def)
+           if (twochem) CALL ef_shift_twochem(npe, dos_ef, dos_ef_cond, ldos, ldos_cond, drhoscfh,&
+                             drhoscfh_cond,dbecsum,dbecsum_cond, becsum1,becsum1_cond, irr, sym_def)
         ELSE
-           CALL ef_shift(npe, dos_ef, ldos, drhoscfh, irr=irr, sym_def=sym_def)
+           CALL ef_shift(npe, dos_ef, ldos, drhoscfh, sym_def=sym_def)
+           if (twochem) CALL ef_shift_twochem(npe, dos_ef,dos_ef_cond, ldos,ldos_cond, drhoscfh,&
+                                           drhoscfh_cond, irr=irr, sym_def=sym_def)
         ENDIF
      ENDIF
      !
@@ -442,7 +469,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      !   Here we symmetrize them ...
      !
      IF (.not.lgamma_gamma) THEN
-        call psymdvscf (npe, irr, drhoscfh)
+        CALL psymdvscf(drhoscfh)
         IF ( noncolin.and.domag ) CALL psym_dmag( npe, irr, drhoscfh)
         IF (okpaw) THEN
            IF (minus_q) CALL PAW_dumqsymmetrize(dbecsum,npe,irr, npertx,irotmq,rtau,xq,tmq)
@@ -462,16 +489,11 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         call zcopy (dfftp%nnr*nspin_mag,drhoscfh(1,1,ipert),1,dvscfout(1,1,ipert),1)
         !
         ! Compute the response of the core charge density
-        ! IT: Should the condition "imode0+ipert > 0" be removed?
         !
-        if (imode0+ipert > 0) then
-           call addcore (imode0+ipert, drhoc)
-        else
-           drhoc(:) = (0.0_DP,0.0_DP)
-        endif
+        call addcore(u(1, imode0+ipert), drhoc)
         !
         ! Compute the response HXC potential
-        call dv_of_drho (dvscfout(1,1,ipert), .true., drhoc)
+        call dv_of_drho (dvscfout(1,1,ipert), drhoc = drhoc)
         !
      enddo
      !
@@ -495,6 +517,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      ENDIF
      !
      IF (lmetq0 .AND. convt) CALL ef_shift_wfc(npe, ldoss, drhoscf)
+     IF (twochem.and. lmetq0 .AND. convt) CALL ef_shift_wfc_twochem(npe, ldoss,ldoss_cond, drhoscf)
      !
      ! check that convergent have been reached on ALL processors in this image
      CALL check_all_convt(convt)
@@ -519,11 +542,11 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         call newdq (dvscfin, npe)
         !
         !  In the noncollinear magnetic case computes the int3 coefficients with
-        !  the opposite sign of the magnetic field. They are saved in int3_save,
+        !  the opposite sign of the magnetic field. They are saved in int3_nc_save,
         !  that must have been allocated by the calling routine
         !
         IF (noncolin.AND.domag) THEN
-           int3_save(:,:,:,:,:,1)=int3_nc(:,:,:,:,:)
+           int3_nc_save(:,:,:,:,:,1)=int3_nc(:,:,:,:,:)
            IF (okpaw) rho%bec(:,:,2:4)=-rho%bec(:,:,2:4)
            DO ipert=1,npe
               dvscfin(:,2:4,ipert)=-dvscfin(:,2:4,ipert)
@@ -538,7 +561,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
            !   here compute the int3 integrals
            !
            CALL newdq (dvscfin, npe)
-           int3_save(:,:,:,:,:,2)=int3_nc(:,:,:,:,:)
+           int3_nc_save(:,:,:,:,:,2)=int3_nc(:,:,:,:,:)
            !
            !  restore the correct sign of the magnetic field.
            !
@@ -550,7 +573,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
            !
            !  put into int3_nc the coefficient with +B
            !
-           int3_nc(:,:,:,:,:)=int3_save(:,:,:,:,:,1)
+           int3_nc(:,:,:,:,:)=int3_nc_save(:,:,:,:,:,1)
         ENDIF
      END IF
      !
@@ -600,7 +623,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         if (elph) call elphel (irr, npe, imode0, dvscfins)
      end if
   endif
-  if (convt.and.nlcc_any) call addnlcc (imode0, drhoscfh, npe)
+  if (convt.and.nlcc_any) call dynmat_nlcc (imode0, drhoscfh, npe)
   !
   CALL apply_dpot_deallocate()
   if (allocated(ldoss)) deallocate (ldoss)
@@ -614,13 +637,25 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   IF (noncolin) deallocate (dbecsum_nc)
   deallocate (dvscfout)
   deallocate (drhoscfh)
+  if (twochem.and.lmetq0) then
+        !deallocate for twochem calculation at gamma
+        if (allocated(ldoss_cond)) deallocate (ldoss_cond)
+        if (allocated(ldos_cond)) deallocate (ldos_cond)
+        deallocate (dbecsum_cond)
+        IF (okpaw) THEN
+                if (allocated(becsum1_cond)) deallocate (becsum1_cond)
+        ENDIF
+        IF (noncolin) deallocate (dbecsum_cond_nc)
+        deallocate (drhoscfh_cond)
+        deallocate (drhoscf_cond)
+  end if
   !$acc exit data delete(dvscfins)
   if (doublegrid) deallocate (dvscfins)
   deallocate (dvscfin)
   deallocate(aux2)
   deallocate(drhoc)
   IF (noncolin.AND.domag.AND.okvan) THEN
-     DEALLOCATE (int3_save)
+     DEALLOCATE (int3_nc_save)
      DEALLOCATE (dbecsum_aux)
   ENDIF
 
