@@ -182,7 +182,6 @@ SUBROUTINE c_phase
    USE mp,                   ONLY : mp_sum
    USE qes_libs_module,      ONLY : qes_reset
    USE qexsd_init,           ONLY : qexsd_init_berryPhaseOutput,  qexsd_bp_obj
-   USE wavefunctions_gpum,   ONLY : using_evc
    USE uspp_init,            ONLY : init_us_2 
 
 !  --- Avoid implicit definitions ---
@@ -288,7 +287,6 @@ SUBROUTINE c_phase
 !  -------------------------------------------------------------------------   !
 !                               INITIALIZATIONS
 !  -------------------------------------------------------------------------   !
-   CALL using_evc(0)           ! Syncronize from gpu data
    ALLOCATE (psi(npwx*npol,nbnd))
    ALLOCATE (aux(ngm*npol))
    ALLOCATE (aux0(ngm*npol))
@@ -473,7 +471,6 @@ SUBROUTINE c_phase
                   npw1 = ngk(kpoint)
                   igk1(:) = igk_k(:,kpoint)
                   CALL get_buffer(evc,nwordwfc,iunwfc,kpoint)
-                  CALL using_evc(1)
                   if (okvan) then
                      CALL init_us_2(npw1,igk1,xk(1,kpoint),vkb)
                      CALL calbec(npw1, vkb, evc, becp_bp)
@@ -483,7 +480,6 @@ SUBROUTINE c_phase
                   npw1 = ngk(kstart)
                   igk1(:) = igk_k(:,kstart)
                   CALL get_buffer(evc,nwordwfc,iunwfc,kstart)
-                  CALL using_evc(1)
                   if (okvan) then
                      CALL init_us_2(npw1,igk1,xk(1,kstart),vkb)
                      CALL calbec(npw1, vkb, evc, becp_bp)
@@ -546,7 +542,6 @@ SUBROUTINE c_phase
 
 !              --- Matrix elements calculation ---
 
-               CALL using_evc(0)
                mat(:,:) = (0.d0, 0.d0)
                DO mb=1,nbnd
                   IF ( l_cal(mb) ) THEN
