@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2022 Quantum ESPRESSO group
+! Copyright (C) 2001-2024 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -32,9 +32,6 @@ SUBROUTINE c_bands( iter )
   USE check_stop,           ONLY : check_stop_now
   USE gcscf_module,         ONLY : lgcscf
   USE add_dmft_occ,         ONLY : dmft, dmft_updated
-
-  USE wavefunctions_gpum,   ONLY : using_evc
-  USE wvfct_gpum,           ONLY : using_et
   USE uspp_init,            ONLY : init_us_2
   USE device_fbuff_m,       ONLY : dev_buf
   !
@@ -57,11 +54,9 @@ SUBROUTINE c_bands( iter )
   !
   !
   CALL start_clock( 'c_bands' ); !write (*,*) 'start c_bands' ; FLUSH(6)
-  CALL using_evc(0)
   !
   ik_ = 0
   avg_iter = 0.D0
-  IF ( restart ) CALL using_et(1)
   IF ( restart ) CALL restart_in_cbands( ik_, ethr, avg_iter, et )
   !
   ! ... If restarting, calculated wavefunctions have to be read from file
@@ -69,9 +64,10 @@ SUBROUTINE c_bands( iter )
   ! ...  directly from file, in order to avoid wasting memory)
   !
   DO ik = 1, ik_
-     IF ( nks > 1 .OR. lelfield ) &
+     IF ( nks > 1 .OR. lelfield ) THEN
         CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
-     IF ( nks > 1 .OR. lelfield ) CALL using_evc(1)
+        !$acc update device(evc)
+     END IF
   ENDDO
   !
   IF ( isolve == 0 ) THEN
@@ -116,9 +112,10 @@ SUBROUTINE c_bands( iter )
      !
      ! ... read in wavefunctions from the previous iteration
      !
-     IF ( nks > 1 .OR. lelfield ) &
+     IF ( nks > 1 .OR. lelfield ) THEN
           CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
-     IF ( nks > 1 .OR. lelfield ) CALL using_evc(2)
+          !$acc update device(evc)
+     END IF
      !
      ! ... Needed for DFT+Hubbard
      !
@@ -130,15 +127,15 @@ SUBROUTINE c_bands( iter )
      !
      IF (.NOT. ( dmft .AND. .NOT. dmft_updated ) ) THEN
         call diag_bands ( iter, ik, avg_iter )
+        !sync evc here to allow later use of converged wavefunction on host
+        !$acc update self(evc)
      END IF
      !
      ! ... save wave-functions to be used as input for the
      ! ... iterative diagonalization of the next scf iteration
      ! ... and for rho calculation
      !
-     CALL using_evc(0)
-     IF ( nks > 1 .OR. lelfield ) &
-          CALL save_buffer ( evc, nwordwfc, iunwfc, ik )
+     IF ( nks > 1 .OR. lelfield ) CALL save_buffer ( evc, nwordwfc, iunwfc, ik )
      !
      ! ... beware: with pools, if the number of k-points on different
      ! ... pools differs, make sure that all processors are still in
@@ -148,7 +145,6 @@ SUBROUTINE c_bands( iter )
      !
      IF (ik <= nkdum) THEN
         IF (check_stop_now()) THEN
-           CALL using_et(0)
            CALL save_in_cbands( ik, ethr, avg_iter, et )
            RETURN
         ENDIF
@@ -219,8 +215,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   USE mp,                   ONLY : mp_sum, mp_bcast
   USE xc_lib,               ONLY : exx_is_active
   USE gcscf_module,         ONLY : lgcscf
-  USE wavefunctions_gpum,   ONLY : evc_d, using_evc, using_evc_d
-  USE wvfct_gpum,           ONLY : et_d, using_et, using_et_d
   !
   USE control_flags,        ONLY : scissor
   USE sci_mod,              ONLY : evcc
@@ -386,11 +380,11 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
              IF ( .NOT. lrot ) THEN
                 !
                 IF (.not. use_gpu) THEN
-                   CALL using_evc(1);  CALL using_et(1); ! et is used as intent(out), set intento=2?
                    CALL rotate_wfc( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
                 ELSE
-                   CALL using_evc_d(1);  CALL using_et_d(1); ! et is used as intent(out), set intento=2?
-                   CALL rotate_wfc_gpu( npwx, npw, nbnd, gstart, nbnd, evc_d, npol, okvan, evc_d, et_d(1,ik) )
+                   !$acc host_data use_device(et)
+                   CALL rotate_wfc_gpu( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
+                   !$acc end host_data
                 END IF
                 !
                 avg_iter = avg_iter + 1.D0
@@ -400,23 +394,23 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           !
           IF ( isolve == 1 ) THEN
              IF (.not. use_gpu) THEN
-                CALL using_evc(1);  CALL using_et(1); CALL using_h_diag(0) ! precontidtion has intent(in)
+                CALL using_h_diag(0) ! precondition has intent(in)
                 CALL rcgdiagg( hs_1psi, s_1psi, h_diag, &
                          npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
              ELSE
-                CALL using_evc_d(1);  CALL using_et_d(1); CALL using_h_diag_d(0) ! precontidtion has intent(in)
+                CALL using_h_diag_d(0) ! precondition has intent(in)
                 CALL rcgdiagg_gpu( hs_1psi_gpu, s_1psi_gpu, h_diag_d, &
-                         npwx, npw, nbnd, evc_d, et_d(1,ik), btype(1,ik), &
+                         npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
-             !
+                !
              END IF
              !
              avg_iter = avg_iter + cg_iter
              !
           ELSE IF ( isolve == 2 ) THEN
              IF (.not. use_gpu) THEN
-               CALL using_evc(1);  CALL using_et(1); CALL using_h_diag(0) ! precontidtion has intent(in)
+               CALL using_h_diag(0) ! precondition has intent(in)
                CALL ppcg_gamma( h_psi, s_psi, okvan, h_diag, &
                            npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                            0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
@@ -424,10 +418,12 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                avg_iter = avg_iter + ppcg_iter
                !
              ELSE
-               CALL using_evc_d(1);  CALL using_et_d(1); CALL using_h_diag_d(0) ! precontidtion has intent(in)
+               CALL using_h_diag_d(0) ! precondition has intent(in)
+               !$acc host_data use_device(evc, et)
                CALL ppcg_gamma_gpu( h_psi_gpu, s_psi_acc, okvan, h_diag_d, &
-                           npwx, npw, nbnd, evc_d, et_d(1,ik), btype(1,ik), &
+                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                            0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
+               !$acc end host_data
                !
                avg_iter = avg_iter + ppcg_iter
                !
@@ -435,7 +431,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           ELSE
              !
              IF (.not. use_gpu ) THEN
-               CALL using_evc(1);  CALL using_et(1); CALL using_h_diag(0) ! precontidtion has intent(in)
+               CALL using_h_diag(0) ! precondition has intent(in)
                CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                !
@@ -443,9 +439,11 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                ! write (6,*) ntry, avg_iter, nhpsi
                !
              ELSE
-               CALL using_evc_d(1);  CALL using_et_d(1); CALL using_h_diag_d(0) ! precontidtion has intent(in)
+               CALL using_h_diag_d(0) ! precondition has intent(in)
+               !$acc host_data use_device(et)
                CALL paro_gamma_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
-                          npwx, npw, nbnd, evc_d, et_d(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+                          npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+               !$acc end host_data
                !
                avg_iter = avg_iter + nhpsi/float(nbnd) 
                ! write (6,*) ntry, avg_iter, nhpsi
@@ -461,25 +459,19 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           IF ( test_exit_cond() ) EXIT  CG_loop
           !
        ENDDO CG_loop
+       !$acc update self(et)
        !
     ELSE IF ( isolve == 4 .AND. .NOT. rmm_use_davidson(iter)) THEN
        !
        ! ... RMM-DIIS diagonalization
        !
-       IF ( .not. use_gpu) THEN
-          ALLOCATE( hevc  ( npwx*npol, nbnd ) )
-          IF ( okvan ) THEN
-             ALLOCATE( sevc( npwx*npol, nbnd ) )
-          ELSE
-             sevc => evc
-          END IF
+       ALLOCATE( hevc  ( npwx*npol, nbnd ) )
+       !$acc enter data create(hevc)
+       IF ( okvan ) THEN
+          ALLOCATE( sevc( npwx*npol, nbnd ) )
+          !$acc enter data create(sevc)
        ELSE
-          ALLOCATE( hevc_d( npwx*npol, nbnd ) )
-          IF ( okvan ) THEN
-             ALLOCATE( sevc_d( npwx*npol, nbnd ) )
-          ELSE
-             sevc_d => evc_d !evc_d allocated in wfcinit_gpu
-          END IF
+          sevc => evc
        END IF
        !
        ntry = 0
@@ -501,7 +493,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
               END FORALL
               !
               IF (.not. use_gpu ) THEN
-                CALL using_evc(1);  CALL using_et(1); CALL using_h_diag(0) ! precontidtion has intent(in)
+                CALL using_h_diag(0) ! precondition has intent(in)
                 CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
                            npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                 !
@@ -509,9 +501,12 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                 ! write (6,*) ntry, avg_iter, nhpsi
                 !
               ELSE
-                CALL using_evc_d(1);  CALL using_et_d(1); CALL using_h_diag_d(0) ! precontidtion has intent(in)
+                CALL using_h_diag_d(0) ! precondition has intent(in)
+                !$acc host_data use_device(et)
                 CALL paro_gamma_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
-                           npwx, npw, nbnd, evc_d, et_d(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+                           npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+                !$acc end host_data
+                !$acc update self(et)
                 !
                 avg_iter = avg_iter + nhpsi/float(nbnd) 
                 ! write (6,*) ntry, avg_iter, nhpsi
@@ -521,14 +516,15 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           ELSE IF ( .NOT. lrot ) THEN
              !
              IF (.not. use_gpu) THEN
-                CALL using_evc(1);  CALL using_et(1); !precontidtion has intent(in)
                 CALL rotate_xpsi( h_psi, s_psi, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
                                evc, hevc, sevc, et(:,ik), USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = .TRUE. )
 #if defined(__CUDA)
              ELSE
-                CALL using_evc_d(1);  CALL using_et_d(1); !precontidtion has intent(in)
-                CALL rotate_xpsi( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc_d, npol, okvan, &
-                               evc_d, hevc_d, sevc_d, et_d(:,ik), USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = .TRUE.)
+                !$acc host_data use_device(et)
+                CALL rotate_xpsi( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
+                               evc, hevc, sevc, et(:,ik), USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = .TRUE.)
+                !$acc end host_data
+                !$acc update self(et)
 #endif
              END IF
              !
@@ -538,15 +534,14 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           !
           !
           IF (.not. use_gpu) THEN
-            CALL using_evc(1);  CALL using_et(1); CALL using_h_diag(0) !precontidtion has intent(in)
+            CALL using_h_diag(0) !precondition has intent(in)
             CALL rrmmdiagg( h_psi, s_psi, npwx, npw, nbnd, evc, hevc, sevc, &
                          et(1,ik), g2kin(1), btype(1,ik), ethr, rmm_ndim, &
                          okvan, lrot, exx_is_active(), notconv, rmm_iter )
           ELSE
-             CALL using_evc_d(1);  CALL using_et(1)!precontidtion has intent(in)
              !$acc data present(g2kin)
              !$acc host_data use_device(g2kin)
-             CALL rrmmdiagg_gpu( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, evc_d, hevc_d, sevc_d, &
+             CALL rrmmdiagg_gpu( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, evc, hevc, sevc, &
                           et(1,ik), g2kin, btype(1,ik), ethr, rmm_ndim, &
                           okvan, lrot, exx_is_active(), notconv, rmm_iter )
              !$acc end host_data 
@@ -569,31 +564,25 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        ! ... Gram-Schmidt orthogonalization
        !
        IF (.not. use_gpu) THEN
-        CALL using_evc(1);  CALL using_et(1); !precontidtion has intent(in)
         CALL gram_schmidt_gamma( npwx, npw, nbnd, evc, hevc, sevc, et(1,ik), &
                         okvan, .TRUE., .TRUE., gs_nblock )
        ELSE
-          CALL using_evc_d(1);  CALL using_et(1); !precontidtion has intent(in)
-          CALL gram_schmidt_gamma_gpu( npwx, npw, nbnd, evc_d, hevc_d, sevc_d, et(1,ik), &
+          !$acc host_data use_device(evc, hevc, sevc)
+          CALL gram_schmidt_gamma_gpu( npwx, npw, nbnd, evc, hevc, sevc, et(1,ik), &
                           okvan, .TRUE., .TRUE., gs_nblock )
+          !$acc end host_data
+          !$acc update device(et)
        END IF
        !
        avg_iter = avg_iter + 0.5D0
        !
-       IF ( .not. use_gpu) THEN 
-          DEALLOCATE( hevc )
-          IF ( okvan ) THEN
-             DEALLOCATE( sevc )
-          ELSE
-             NULLIFY( sevc )
-          END IF
-        ELSE
-          DEALLOCATE( hevc_d )
-          IF ( okvan ) THEN
-             DEALLOCATE( sevc_d )
-          ELSE
-             NULLIFY( sevc_d )
-          END IF
+       !$acc exit data delete(hevc)
+       DEALLOCATE( hevc )
+       IF ( okvan ) THEN
+          !$acc exit data delete(sevc)
+          DEALLOCATE( sevc )
+       ELSE
+          NULLIFY( sevc )
        END IF
        !
     ELSE
@@ -636,7 +625,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           lrot = ( iter == 1 )
           !
           IF (.not. use_gpu) THEN
-             CALL using_evc(1); CALL using_et(1);
              IF ( use_para_diag ) THEN
 !                ! make sure that all processors have the same wfc
                 CALL pregterg( h_psi, s_psi, okvan, g_psi, &
@@ -647,21 +635,24 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                          npw, npwx, nbnd, nbndx, evc, ethr, &
                          et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call
              END IF
-             ! CALL using_evc(1) done above
+             ! 
           ELSE
-             CALL using_evc_d(1); CALL using_et_d(1);
+             !$acc host_data use_device(et)
              IF ( use_para_diag ) THEN
+                !$acc host_data use_device(evc)
                 CALL pregterg_gpu( h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
-                            npw, npwx, nbnd, nbndx, evc_d, ethr, &
-                            et_d(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call 
+                            npw, npwx, nbnd, nbndx, evc, ethr, &
+                            et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call 
+                !$acc end host_data
                 !
              ELSE
                 !
                 CALL regterg (  h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
-                         npw, npwx, nbnd, nbndx, evc_d, ethr, &
-                         et_d(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call
+                         npw, npwx, nbnd, nbndx, evc, ethr, &
+                         et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi ) !    BEWARE gstart has been removed from call
              END IF
-             ! CALL using_evc_d(1) ! done above
+             !$acc end host_data
+             !$acc update self(et)
           END IF
           !
           avg_iter = avg_iter + dav_iter
@@ -676,7 +667,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
     ENDIF
     !
-    !
     RETURN
     !
   END SUBROUTINE diag_bands_gamma
@@ -700,7 +690,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        !
        ! ... save wave functions from previous iteration for electric field
        !
-       CALL using_evc(0)
        evcel = evc
        !
        !... read projectors from disk
@@ -764,11 +753,11 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
              IF ( .NOT. lrot ) THEN
                 !
                 IF ( .not. use_gpu ) THEN
-                   CALL using_evc(1); CALL using_et(1);
                    CALL rotate_wfc( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
                 ELSE
-                   CALL using_evc_d(1); CALL using_et_d(1);
-                   CALL rotate_wfc_gpu( npwx, npw, nbnd, gstart, nbnd, evc_d, npol, okvan, evc_d, et_d(1,ik) )
+                   !$acc host_data use_device(et)
+                   CALL rotate_wfc_gpu( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
+                   !$acc end host_data
                 END IF
                 !
                 avg_iter = avg_iter + 1.D0
@@ -777,14 +766,14 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           !
           IF ( isolve == 1) then
              IF ( .not. use_gpu ) THEN
-                CALL using_evc(1); CALL using_et(1); CALL using_h_diag(0)
+                CALL using_h_diag(0)
                 CALL ccgdiagg( hs_1psi, s_1psi, h_diag, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
              ELSE
-                CALL using_evc_d(1); CALL using_et_d(1); CALL using_h_diag_d(0)
+                CALL using_h_diag_d(0)
                 CALL ccgdiagg_gpu( hs_1psi_gpu, s_1psi_gpu, h_diag_d, &
-                         npwx, npw, nbnd, npol, evc_d, et_d(1,ik), btype(1,ik), &
+                         npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
              END IF
              !
@@ -792,7 +781,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
              !
           ELSE IF ( isolve == 2) then
              IF ( .not. use_gpu ) THEN
-               CALL using_evc(1); CALL using_et(1); CALL using_h_diag(0)
+               CALL using_h_diag(0)
                ! BEWARE npol should be added to the arguments
                CALL ppcg_k( h_psi, s_psi, okvan, h_diag, &
                            npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
@@ -801,11 +790,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                avg_iter = avg_iter + ppcg_iter
                !
              ELSE
-               CALL using_evc_d(1); CALL using_et_d(1); CALL using_h_diag_d(0)
+               CALL using_h_diag_d(0)
                ! BEWARE npol should be added to the arguments
+               !$acc host_data use_device(evc, et)
                CALL ppcg_k_gpu( h_psi_gpu, s_psi_acc, okvan, h_diag_d, &
-                           npwx, npw, nbnd, npol, evc_d, et_d(1,ik), btype(1,ik), &
+                           npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
                            0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
+               !$acc end host_data
                !
                avg_iter = avg_iter + ppcg_iter
                !
@@ -813,16 +804,18 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           ELSE 
              !
              IF ( .not. use_gpu ) THEN
-               CALL using_evc(1); CALL using_et(1); CALL using_h_diag(0)
+               CALL using_h_diag(0)
                CALL paro_k_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
                         npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                !
                avg_iter = avg_iter + nhpsi/float(nbnd) 
                ! write (6,*) ntry, avg_iter, nhpsi
              ELSE
-               CALL using_evc_d(1); CALL using_et_d(1); CALL using_h_diag_d(0)
+               CALL using_h_diag_d(0)
+               !$acc host_data use_device(et)
                CALL paro_k_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
-                        npwx, npw, nbnd, npol, evc_d, et_d(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+                        npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+               !$acc end host_data
                !
                avg_iter = avg_iter + nhpsi/float(nbnd) 
                ! write (6,*) ntry, avg_iter, nhpsi
@@ -836,26 +829,20 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           IF ( test_exit_cond() ) EXIT  CG_loop
           !
        ENDDO CG_loop
+       !$acc update self(et)
        !
     ELSE IF ( isolve == 4 .AND. .NOT. rmm_use_davidson(iter) )  THEN
        !
        ! ... RMM-DIIS diagonalization
        !
-       IF ( .not. use_gpu) THEN 
-         ALLOCATE( hevc( npwx*npol, nbnd ) )
-         IF ( okvan ) THEN
-            ALLOCATE( sevc( npwx*npol, nbnd ) )
-         ELSE
-            sevc => evc
-         END IF
+       ALLOCATE( hevc( npwx*npol, nbnd ) )
+       !$acc enter data create(hevc)
+       IF ( okvan ) THEN
+          ALLOCATE( sevc( npwx*npol, nbnd ) )
+          !$acc enter data create(sevc)
        ELSE
-         ALLOCATE( hevc_d( npwx*npol, nbnd ) )
-         IF ( okvan ) THEN
-            ALLOCATE( sevc_d( npwx*npol, nbnd ) )
-         ELSE
-            sevc_d => evc_d !evc_d allocated in wfcinit_gpu
-         END IF
-       END IF  
+          sevc => evc
+       END IF
        !
        ntry = 0
        !
@@ -876,16 +863,18 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
               CALL usnldiag(npw, h_diag, s_diag )
               !
               IF ( .not. use_gpu ) THEN
-                CALL using_evc(1); CALL using_et(1)
                 CALL paro_k_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
                 !
                 avg_iter = avg_iter + nhpsi/float(nbnd) 
                 ! write (6,*) ntry, avg_iter, nhpsi
               ELSE
-                CALL using_evc_d(1); CALL using_et_d(1); CALL using_h_diag_d(0)
+                CALL using_h_diag_d(0)
+                !$acc host_data use_device(et)
                 CALL paro_k_new( h_psi_gpu, s_psi_acc, hs_psi_gpu, g_1psi_gpu, okvan, &
-                         npwx, npw, nbnd, npol, evc_d, et_d(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+                         npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+                !$acc end host_data
+                !$acc update self(et)
                 !
                 avg_iter = avg_iter + nhpsi/float(nbnd) 
                 ! write (6,*) ntry, avg_iter, nhpsi
@@ -895,16 +884,17 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           ELSE IF ( .NOT. lrot ) THEN
              !
              IF ( .not. use_gpu ) THEN
-                CALL using_evc(1);  CALL using_et(1); !precontidtion has intent(in)
                 CALL rotate_xpsi( h_psi, s_psi, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
                                   evc, hevc, sevc, et(:,ik), & 
                                   USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = gamma_only )
 #if defined(__CUDA)
              ELSE
-                CALL using_evc_d(1);  CALL using_et_d(1); !precontidtion has intent(in)
-                CALL rotate_xpsi( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc_d, npol, okvan, &
-                                  evc_d, hevc_d, sevc_d, et_d(:,ik), &
+                !$acc host_data use_device(et)
+                CALL rotate_xpsi( h_psi, s_psi, h_psi_gpu, s_psi_acc, npwx, npw, nbnd, nbnd, evc, npol, okvan, &
+                                  evc, hevc, sevc, et(:,ik), &
                                   USE_PARA_DIAG = use_para_diag, GAMMA_ONLY = gamma_only )
+                !$acc end host_data
+                !$acc update self(et)
 #endif
              END IF
              !
@@ -913,15 +903,14 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           END IF
           !
           IF ( .not. use_gpu ) THEN
-             CALL using_evc(1); CALL using_et(1); CALL using_h_diag(0)
+             CALL using_h_diag(0)
              CALL crmmdiagg( h_psi, s_psi, npwx, npw, nbnd, npol, evc, hevc, sevc, &
                              et(1,ik), g2kin(1), btype(1,ik), ethr, rmm_ndim, &
                              okvan, lrot, exx_is_active(), notconv, rmm_iter )
           ELSE
-             CALL using_evc_d(1); CALL using_et(1) 
              !$acc data present(g2kin)
              !$acc host_data use_device(g2kin)
-             CALL crmmdiagg_gpu( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, npol, evc_d, hevc_d, sevc_d, &
+             CALL crmmdiagg_gpu( h_psi_gpu, s_psi_acc, npwx, npw, nbnd, npol, evc, hevc, sevc, &
                              et(1,ik), g2kin(1), btype(1,ik), ethr, rmm_ndim, &
                              okvan, lrot, exx_is_active(), notconv, rmm_iter )
              !$acc end host_data
@@ -943,33 +932,26 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
        ! ... Gram-Schmidt orthogonalization
        !
        IF ( .not. use_gpu ) THEN
-          CALL using_evc(1); CALL using_et(1);
           CALL gram_schmidt_k( npwx, npw, nbnd, npol, evc, hevc, sevc, et(1,ik), &
                              okvan, .TRUE., .TRUE., gs_nblock )
        ELSE
-          CALL using_evc_d(1); CALL using_et(1); 
-          CALL gram_schmidt_k_gpu( npwx, npw, nbnd, npol, evc_d, hevc_d, sevc_d, et(1,ik), &
+          !$acc host_data use_device(evc, hevc, sevc)
+          CALL gram_schmidt_k_gpu( npwx, npw, nbnd, npol, evc, hevc, sevc, et(1,ik), &
                              okvan, .TRUE., .TRUE., gs_nblock )
-
+          !$acc end host_data
+          !$acc update device(et)
        END IF
        !
        avg_iter = avg_iter + 0.5D0
        !
-       IF ( .not. use_gpu) THEN 
-         DEALLOCATE( hevc )
-         IF ( okvan ) THEN
-            DEALLOCATE( sevc )
-         ELSE
-            NULLIFY( sevc )
-         END IF
+       !$acc exit data delete(hevc)
+       DEALLOCATE( hevc )
+       IF ( okvan ) THEN
+          !$acc exit data delete(sevc)
+          DEALLOCATE( sevc )
        ELSE
-         DEALLOCATE( hevc_d )
-         IF ( okvan ) THEN
-            DEALLOCATE( sevc_d )
-         ELSE
-            NULLIFY( sevc_d )
-         END IF
-       END IF 
+          NULLIFY( sevc )
+       END IF
        !
        !
     ELSE
@@ -1022,7 +1004,6 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
           lrot = ( iter == 1 )
           !
           IF (.not. use_gpu ) THEN
-             CALL using_evc(1) ; CALL using_et(1)
              IF ( use_para_diag ) then
                 !
                 CALL pcegterg( h_psi, s_psi, okvan, g_psi, &
@@ -1036,20 +1017,23 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
                                et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
              END IF
           ELSE
-             CALL using_evc_d(1) ; CALL using_et_d(1) 
+             !$acc host_data use_device(et)
              IF ( use_para_diag ) then
                 !
+                !$acc host_data use_device(evc)
                 CALL pcegterg_gpu( h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
-                               npw, npwx, nbnd, nbndx, npol, evc_d, ethr, &
-                               et_d(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
-
+                               npw, npwx, nbnd, nbndx, npol, evc, ethr, &
+                               et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+                !$acc end host_data
                 !
              ELSE
                 !
                 CALL cegterg ( h_psi_gpu, s_psi_acc, okvan, g_psi_gpu, &
-                               npw, npwx, nbnd, nbndx, npol, evc_d, ethr, &
-                               et_d(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+                               npw, npwx, nbnd, nbndx, npol, evc, ethr, &
+                               et(1, ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
              END IF
+             !$acc end host_data 
+             !$acc update self(et)
           END IF
           !
           avg_iter = avg_iter + dav_iter
@@ -1177,7 +1161,7 @@ SUBROUTINE c_bands_nscf( )
   USE io_global,            ONLY : stdout
   USE io_files,             ONLY : iunhub, iunwfc, nwordwfc, nwordwfcU
   USE buffers,              ONLY : get_buffer, save_buffer, close_buffer
-  USE basis,                ONLY : starting_wfc
+  USE starting_scf,         ONLY : starting_wfc
   USE klist,                ONLY : nkstot, nks, xk, ngk, igk_k
   USE uspp,                 ONLY : vkb, nkb
   USE gvect,                ONLY : g
@@ -1189,8 +1173,6 @@ SUBROUTINE c_bands_nscf( )
   USE mp_pools,             ONLY : npool, kunit, inter_pool_comm
   USE mp,                   ONLY : mp_sum
   USE check_stop,           ONLY : check_stop_now
-  USE wavefunctions_gpum,   ONLY : using_evc
-  USE wvfct_gpum,           ONLY : using_et
   USE uspp_init,            ONLY : init_us_2
   IMPLICIT NONE
   !
@@ -1210,12 +1192,10 @@ SUBROUTINE c_bands_nscf( )
   !
   ik_ = 0
   avg_iter = 0.D0
-  IF ( restart ) CALL using_et(1)
   IF ( restart ) CALL restart_in_cbands( ik_, ethr, avg_iter, et )
   !
   ! ... If restarting, calculated wavefunctions have to be read from file
   !
-  CALL using_evc(1)
   DO ik = 1, ik_
      CALL get_buffer( evc, nwordwfc, iunwfc, ik )
   ENDDO
@@ -1268,7 +1248,6 @@ SUBROUTINE c_bands_nscf( )
      !
      IF ( TRIM(starting_wfc) == 'file' ) THEN
         !
-        CALL using_evc(1)
         CALL get_buffer( evc, nwordwfc, iunwfc, ik )
         !
      ELSE
@@ -1280,10 +1259,10 @@ SUBROUTINE c_bands_nscf( )
      ! ... diagonalization of bands for k-point ik
      !
      CALL diag_bands( 1, ik, avg_iter )
+     !$acc update self(evc)
      !
      ! ... save wave-functions (unless disabled in input)
      !
-     IF ( io_level > -1 ) CALL using_evc(0)
      IF ( io_level > -1 ) CALL save_buffer( evc, nwordwfc, iunwfc, ik )
      !
      ! ... beware: with pools, if the number of k-points on different
@@ -1297,7 +1276,6 @@ SUBROUTINE c_bands_nscf( )
         ! ... save wavefunctions to file
         !
         IF ( check_stop_now() ) THEN
-           CALL using_et(0)
            CALL save_in_cbands( ik, ethr, avg_iter, et )
            RETURN
         ENDIF
