@@ -1642,6 +1642,10 @@ MODULE realus
     !
     REAL(DP), EXTERNAL :: ddot
     !
+    !-------------------TEMPORARY-----------
+    INTEGER :: ngk1
+    LOGICAL :: is_present, acc_is_present
+    !---------------------------------------    
     !
     CALL start_clock( 'calbec_rs' )
     !
@@ -1653,9 +1657,18 @@ MODULE realus
     !
     becp_r(:,ibnd)=0.d0
     IF ( ibnd+1 <= last ) becp_r(:,ibnd+1)=0.d0
+    !
     ! Clearly for an odd number of bands for ibnd=nbnd=last you don't have
     ! anymore bands, and so the imaginary part equal zero
     !
+!-------------TEMPORARY---------------------
+    ngk1 = SIZE(psic)
+#if defined(_OPENACC)
+    is_present = acc_is_present(psic,ngk1)
+    !$acc update self(psic) if (is_present)
+#endif
+!-------------------------------------------    
+!
 !
 ! copy psic into a box-friendly array (and scatter it across intra_bbox_comm if needed)
 !
@@ -1677,7 +1690,7 @@ MODULE realus
              !
              mbia = maxbox_beta(ia) ; IF ( mbia == 0 ) CYCLE
              !
-             ijkb0 = ofsbeta(ia)
+             ijkb0 = ofsbeta(ia)             
              !$omp parallel default(shared) private(ih,ikb,ir,bcr,bci)
              !$omp do 
              DO ir =1, mbia
@@ -1703,7 +1716,7 @@ MODULE realus
                 DO ih = 1, nh_nt
                    !
                    ikb = ijkb0 + ih
-                   bci = ddot( mbia, betasave(box_s(ia):box_e(ia),ih), 1, wi(:) , 1 )
+                   bci = ddot( mbia, betasave(box_s(ia): box_e(ia),ih), 1, wi(:) , 1 )
                    becp_r(ikb,ibnd+1) = fac * bci
                    !
                 ENDDO
@@ -2197,15 +2210,13 @@ MODULE realus
     !
     INTEGER :: ebnd
     !
-    !-------------------TEMPORARY-----------
-    INTEGER :: ngk1
-    LOGICAL :: is_present, acc_is_present
-    !---------------------------------------
-    !
     !Task groups
     !
     !The new task group version based on vloc_psi
     !print *, "->Real space"
+    !
+    !$acc data present_or_copyin(orbital) present_or_copyout(psic)
+    !
     CALL start_clock( 'invfft_orbital' )
     !
     IF( dffts%has_task_groups ) THEN
@@ -2226,14 +2237,6 @@ MODULE realus
         !
         CALL wave_g2r( orbital(1:ngk(1),ibnd:ebnd), psic, dffts )
         !
-!-------------TEMPORARY---------------------
-        ngk1 = SIZE(psic)
-#if defined(_OPENACC)
-        is_present = acc_is_present(psic,ngk1)
-        !$acc update self(psic) if (is_present)
-#endif
-!-------------------------------------------
-        !
         IF (PRESENT(conserved)) THEN
           IF (conserved) THEN
             IF (.NOT. ALLOCATED(psic_temp) ) ALLOCATE( psic_temp(SIZE(psic)) )
@@ -2245,6 +2248,8 @@ MODULE realus
     !
     CALL stop_clock( 'invfft_orbital' )
     !
+    !$acc end data
+    !
   END SUBROUTINE invfft_orbital_gamma
   !
   !--------------------------------------------------------------------------
@@ -2289,15 +2294,12 @@ MODULE realus
     LOGICAL :: add_to_orbital_
     COMPLEX(DP), ALLOCATABLE :: psio(:,:)
     !
-!-------------------TEMPORARY-----------
-    INTEGER :: ngk1
-    LOGICAL :: is_present, acc_is_present
-!---------------------------------------
-    !
     ! ... Task groups
     !print *, "->Fourier space"
     CALL start_clock( 'fwfft_orbital' )
     !
+    !$acc data present_or_copyin(psic) present_or_copy(orbital)
+    !
     add_to_orbital_=.FALSE. ; IF( PRESENT(add_to_orbital)) add_to_orbital_ = add_to_orbital
     !
     ! ... New task_groups versions
@@ -2351,31 +2353,29 @@ MODULE realus
         !
         CALL wave_r2g( psic(1:dffts%nnr), psio, dffts )
         !
-!-------------TEMPORARY---------------------
-        ngk1 = SIZE(psic)
-#if defined(_OPENACC)
-        is_present = acc_is_present(psic,ngk1)
-        !$acc update self(psic) if (is_present)
-#endif
-!-------------------------------------------
-        !
         fac = 1.d0
         IF ( ibnd<last ) fac = 0.5d0
         !
         IF ( add_to_orbital_ ) THEN
+#if defined(_OPENACC)                
+           !$acc parallel loop
+#else
            !$omp parallel do
+#endif
            DO j = 1, ngk(1)
               orbital(j,ibnd) = orbital(j,ibnd) + fac*psio(j,1)
               IF (ibnd<last) orbital(j,ibnd+1) = orbital(j,ibnd+1) + fac*psio(j,2)
            ENDDO
-           !$omp end parallel do
         ELSE
+#if defined(_OPENACC)                
+           !$acc parallel loop
+#else
            !$omp parallel do
+#endif                
            DO j = 1, ngk(1)
               orbital(j,ibnd) = fac*psio(j,1)
               IF (ibnd<last) orbital(j,ibnd+1) = fac*psio(j,2)
            ENDDO
-           !$omp end parallel do
         ENDIF
         !
         DEALLOCATE( psio )
@@ -2388,6 +2388,8 @@ MODULE realus
         !
     ENDIF
     !
+    !$acc end data
+    !
     CALL stop_clock( 'fwfft_orbital' )
     !
   END SUBROUTINE fwfft_orbital_gamma
@@ -2611,7 +2613,6 @@ MODULE realus
     USE fft_base,      ONLY : dffts
     USE scf,           ONLY : vrs
     USE lsda_mod,      ONLY : current_spin
-    USE scf_gpum,      ONLY : using_vrs
     !
     IMPLICIT NONE
     !
@@ -2627,8 +2628,6 @@ MODULE realus
     !
     CALL start_clock( 'v_loc_psir' )
 
-    CALL using_vrs(0) ! tg_gather (intent: in)
-
     IF( dffts%has_task_groups ) THEN
         IF (ibnd == 1 ) THEN
           CALL tg_gather( dffts, vrs(:,current_spin), tg_v )
@@ -2670,7 +2669,6 @@ MODULE realus
     USE fft_base,      ONLY : dffts
     USE scf,           ONLY : vrs
     USE lsda_mod,      ONLY : current_spin
-    USE scf_gpum,      ONLY : using_vrs
     !
     IMPLICIT NONE
     !
@@ -2686,8 +2684,6 @@ MODULE realus
     !
     CALL start_clock( 'v_loc_psir' )
 
-    CALL using_vrs(0) ! tg_gather (intent: in)
-
     IF( dffts%has_task_groups ) THEN
         IF (ibnd == 1 ) THEN
           CALL tg_gather( dffts, vrs(:,current_spin), tg_v )
