@@ -66,6 +66,10 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
 #if defined (__ENVIRON)
   USE plugin_flags,         ONLY : use_environ
   USE environ_td_module,    ONLY : calc_environ_dpotential
+#endif
+  !
+#if defined(__CUDA)
+    USE cublas
 #endif
   !
   IMPLICIT NONE
@@ -82,26 +86,30 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
                            & w1(:), w2(:)
   COMPLEX(DP), ALLOCATABLE :: dvrs_temp(:,:), spsi1(:,:), dvrsc(:,:), &
                               & dvrssc(:), sevc1_new(:,:,:)
+  INTEGER :: nnr_siz
   !
   IF (lr_verbosity > 5) THEN
      WRITE(stdout,'("<lr_apply_liouvillian>")')
   ENDIF
   !
-  CALL start_clock('lr_apply')
+  CALL start_clock_gpu('lr_apply')
   !
-  IF (interaction)      CALL start_clock('lr_apply_int')
-  IF (.not.interaction) CALL start_clock('lr_apply_no')
+  IF (interaction)      CALL start_clock_gpu('lr_apply_int')
+  IF (.not.interaction) CALL start_clock_gpu('lr_apply_no')
   !
   ALLOCATE( d_deeq(nhm, nhm, nat, nspin) )
-  d_deeq(:,:,:,:)=0.0d0
-  !
   ALLOCATE( spsi1(npwx, nbnd) )
+  ALLOCATE( sevc1_new(npwx*npol,nbnd,nks))
+  !
+  nnr_siz= dffts%nnr
+!$acc data present_or_copyin(evc1(1:npwx*npol,1:nbnd,1:nks)) present_or_copyout(evc1_new(1:npwx*npol,1:nbnd,1:nks)) create(sevc1_new(1:npwx*npol,1:nbnd,1:nks), spsi1(1:npwx, 1:nbnd)) present_or_copyin(revc0(1:nnr_siz,1:nbnd,1))
+  !
+  d_deeq(:,:,:,:)=0.0d0
+  !$acc kernels
   spsi1(:,:)=(0.0d0,0.0d0)
-  !
-  ALLOCATE(sevc1_new(npwx*npol,nbnd,nks))
   sevc1_new(:,:,:) = (0.0d0,0.0d0)
-  !
   evc1_new(:,:,:) = (0.0d0,0.0d0)
+  !$acc end kernels
   !
   IF ( interaction ) THEN 
      !
@@ -155,7 +163,7 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
            !
            DEALLOCATE ( dvrs )  ! to save memory
            !
-           CALL dv_of_drho(dvrs_temp,.FALSE.)
+           CALL dv_of_drho(dvrs_temp)
            !
            ALLOCATE ( dvrs(dfftp%nnr, nspin) )
            !
@@ -171,7 +179,7 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
            !
            dvrsc(:,1) = rho_1c(:,1)
            !
-           CALL dv_of_drho(dvrsc,.FALSE.)
+           CALL dv_of_drho(dvrsc)
            !
         ENDIF
         !
@@ -232,8 +240,10 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
      ! Here we add the two terms: 
      ! [H(k) - E(k)] * evc1(k) + dV_HXC * revc0(k)
      !
-     CALL zaxpy(size_evc,CMPLX(1.0d0,0.0d0,kind=DP),&
+     !$acc host_data use_device(evc1_new, sevc1_new)        
+     CALL zaxpy(size_evc,CMPLX(1.0d0,0.0d0,kind=DP),&        
                        & evc1_new(:,:,:), 1, sevc1_new(:,:,:), 1)
+     !$acc end host_data          
      !
   ELSEIF ( interaction .and. ltammd ) THEN
      !
@@ -244,8 +254,10 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
      !
      !   Here evc1_new contains the interaction
      !
+     !$acc host_data use_device(evc1_new, sevc1_new)
      CALL zaxpy(size_evc,CMPLX(0.5d0,0.0d0,kind=DP),&
                        & evc1_new(:,:,:) , 1, sevc1_new(:,:,:),1)
+     !$acc end host_data           
      !
   ELSE
      !
@@ -256,21 +268,27 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
      !
   ENDIF
   !
-  IF (gstart == 2 .AND. gamma_only ) sevc1_new(1,:,:) = &
-                & CMPLX( REAL( sevc1_new(1,:,:), DP ), 0.0d0, DP )
   !
-  IF (gstart==2 .and. gamma_only) THEN
-     DO ik=1,nks
-        DO ibnd=1,nbnd
-           IF (abs(aimag(sevc1_new(1,ibnd,ik)))>1.0d-12) THEN
-              !
-              CALL errore(' lr_apply_liouvillian ',&
-                   'Imaginary part of G=0 '// &
-                   'component does not equal zero',1)
-           ENDIF
-        ENDDO
-     ENDDO
+  IF (gstart == 2 .AND. gamma_only ) THEN 
+     !$acc kernels
+     sevc1_new(1,:,:) = &
+                & CMPLX( REAL( sevc1_new(1,:,:), DP ), 0.0d0, DP )
+     !$acc end kernels
   ENDIF
+
+  !
+!  IF (gstart==2 .and. gamma_only) THEN
+!     DO ik=1,nks
+!        DO ibnd=1,nbnd
+!           IF (abs(aimag(sevc1_new(1,ibnd,ik)))>1.0d-12) THEN
+!              !
+!              CALL errore(' lr_apply_liouvillian ',&
+!                   'Imaginary part of G=0 '// &
+!                   'component does not equal zero',1)
+!           ENDIF
+!        ENDDO
+!     ENDDO
+!  ENDIF
   !
   ! Apply the projector on empty states P_c^+.
   ! Note: The projector P_c^+ can be applied only
@@ -285,13 +303,19 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
   ! [H(k) - E(k)] * evc1(k). Keep this in mind if you want to
   ! generalize this subroutine to metals. 
   !
+  !$acc data copyin(evc0)
   DO ik = 1, nks
      !
+
      CALL orthogonalize(sevc1_new(:,:,ik), evc0(:,:,ik), ik, ik, &
                                   & sevc0(:,:,ik), ngk(ik), .true.)
+     !$acc kernels                     
      sevc1_new(:,:,ik) = -sevc1_new(:,:,ik)
+     !$acc end kernels
      !
   ENDDO 
+  !$acc end data
+  !
   !
   ! Here we apply the S^{-1} operator.
   ! See equations after Eq.(47) of B. Walker et al., J. Chem. Phys.
@@ -305,16 +329,17 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, interaction )
                        & sevc1_new(1,1,ik), evc1_new(1,1,ik))
   ENDDO
   !
+  !$acc end data
   IF (allocated(dvrs)) DEALLOCATE(dvrs)
   IF (allocated(dvrss)) DEALLOCATE(dvrss)
   DEALLOCATE(d_deeq)
   DEALLOCATE(spsi1)
   DEALLOCATE(sevc1_new)
   !
-  IF (interaction)      CALL stop_clock('lr_apply_int')
-  IF (.not.interaction) CALL stop_clock('lr_apply_no')
+  IF (interaction)      CALL stop_clock_gpu('lr_apply_int')
+  IF (.not.interaction) CALL stop_clock_gpu('lr_apply_no')
   !
-  CALL stop_clock('lr_apply')
+  CALL stop_clock_gpu('lr_apply')
   !
   RETURN
   !
@@ -330,13 +355,17 @@ CONTAINS
     USE mp_global,                ONLY : ibnd_start, ibnd_end, inter_bgrp_comm
     USE mp,                       ONLY : mp_sum
     USE lr_exx_kernel,            ONLY : lr_exx_sum_int
+#if defined(__CUDA)
+    USE cublas
+#endif
     !
     IMPLICIT NONE
     !
     REAL(DP), ALLOCATABLE :: becp2(:,:)
     REAL(DP), ALLOCATABLE :: tg_dvrss(:)
-    INTEGER :: v_siz, incr, ioff
-    INTEGER :: ibnd_start_gamma, ibnd_end_gamma
+    COMPLEX(DP)           :: coef
+    INTEGER :: v_siz, incr, ioff, ir, nnr_siz
+    INTEGER :: ibnd_start_gamma, ibnd_end_gamma, ibnd, ngk1
     !
     incr = 2
     !
@@ -350,9 +379,13 @@ CONTAINS
     ! Now apply to the ground state wavefunctions
     ! and convert to real space
     !
+    nnr_siz= dffts%nnr
+    !$acc data create (psic(1:nnr_siz))
+    !
     IF ( interaction ) THEN
        !
-       CALL start_clock('interaction')
+       !
+       CALL start_clock_gpu('interaction')
        !
        IF (nkb > 0 .and. okvan) THEN
           ! calculation of becp2
@@ -393,23 +426,25 @@ CONTAINS
           ENDDO
           !end: calculation of becp2
        ENDIF
-
-      IF ( dffts%has_task_groups ) THEN
-         !
-         v_siz =  dffts%nnr_tg
-         !
-         incr = 2 * fftx_ntgrp(dffts)
-         !
-         ALLOCATE( tg_dvrss(1:v_siz) )
-         tg_dvrss=0.0d0
-         !
-         CALL tg_gather(dffts, dvrss, tg_dvrss)
-         !
-      ENDIF
+       !
+       IF ( dffts%has_task_groups ) THEN
+          !
+          v_siz =  dffts%nnr_tg
+          !
+          incr = 2 * fftx_ntgrp(dffts)
+          !
+          ALLOCATE( tg_dvrss(1:v_siz) )
+          tg_dvrss=0.0d0
+          !
+          CALL tg_gather(dffts, dvrss, tg_dvrss)
+          !
+       ENDIF
        !
        ! evc1_new is used as a container for the interaction
        !
+       !$acc kernels
        evc1_new(:,:,:) = (0.0d0,0.0d0)
+       !$acc end kernels
        !
        ibnd_start_gamma = ibnd_start
        IF (MOD(ibnd_start, 2)==0) ibnd_start_gamma = ibnd_start + 1
@@ -417,8 +452,8 @@ CONTAINS
        !
        IF (lr_exx) CALL lr_exx_sum_int()
        !
+       !$acc enter data copyin(dvrss(1:nnr_siz))
        DO ibnd = ibnd_start_gamma ,ibnd_end_gamma, incr
-!      DO ibnd = 1,nbnd,2
           !
           ! Product with the potential vrs = (vltot+vr)
           ! revc0 is on smooth grid. psic is used up to smooth grid
@@ -433,7 +468,9 @@ CONTAINS
              !
           ELSE
              !
-             DO ir = 1,dffts%nnr
+             !DO ir = 1,dffts%nnr
+             !$acc parallel loop
+             DO ir = 1, nnr_siz
                 !
                 psic(ir) = revc0(ir,ibnd,1)*CMPLX(dvrss(ir),0.0d0,DP)
                 !
@@ -446,65 +483,65 @@ CONTAINS
           ENDIF
           !
           IF (real_space .and. okvan .and. nkb > 0) THEN
-          !THE REAL SPACE PART (modified from s_psi)
-                  !fac = sqrt(omega)
-                  !
-                  ijkb0 = 0
-                  iqs = 0
-                  jqs = 0
-                  !
-                  DO nt = 1, ntyp
-                  !
-                    DO ia = 1, nat
+             !THE REAL SPACE PART (modified from s_psi)
+             !fac = sqrt(omega)
+             !
+             ijkb0 = 0
+             iqs = 0
+             jqs = 0
+             !
+             DO nt = 1, ntyp
+                !
+                DO ia = 1, nat
+                   !
+                   IF ( ityp(ia) == nt ) THEN
                       !
-                      IF ( ityp(ia) == nt ) THEN
-                        !
-                        mbia = maxbox_beta(ia)
-                        ALLOCATE( w1(nh(nt)),  w2(nh(nt)) )
-                        w1 = 0.D0
-                        w2 = 0.D0
-                        !
-                        DO ih = 1, nh(nt)
-                        !
-                          DO jh = 1, nh(nt)
-                          !
-                          jkb = ijkb0 + jh
-                          w1(ih) = w1(ih) + becp2(jkb, ibnd)
-                          IF ( ibnd+1 <= nbnd ) w2(ih) = w2(ih) + &
+                      mbia = maxbox_beta(ia)
+                      ALLOCATE( w1(nh(nt)),  w2(nh(nt)) )
+                      w1 = 0.D0
+                      w2 = 0.D0
+                      !
+                      DO ih = 1, nh(nt)
+                         !
+                         DO jh = 1, nh(nt)
+                            !
+                            jkb = ijkb0 + jh
+                            w1(ih) = w1(ih) + becp2(jkb, ibnd)
+                            IF ( ibnd+1 <= nbnd ) w2(ih) = w2(ih) + &
                                &becp2(jkb, ibnd+1)
-                          !
-                          ENDDO
-                        !
-                        ENDDO
-                        !
-                        !w1 = w1 * fac
-                        !w2 = w2 * fac
-                        ijkb0 = ijkb0 + nh(nt)
-                        !
-                        DO ih = 1, nh(nt)
-                          !
-                          DO ir = 1, mbia
-                          !
-                           iqs = jqs + ir
-                           psic( box_beta(box0(ia)+ir) ) = &
+                            !
+                         ENDDO
+                         !
+                      ENDDO
+                      !
+                      !w1 = w1 * fac
+                      !w2 = w2 * fac
+                      ijkb0 = ijkb0 + nh(nt)
+                      !
+                      DO ih = 1, nh(nt)
+                         !
+                         DO ir = 1, mbia
+                            !
+                            iqs = jqs + ir
+                            psic( box_beta(box0(ia)+ir) ) = &
                                 &psic( box_beta(box0(ia)+ir) ) + &
                                 &betasave(box0(ia)+ir,ih)*&
                                 &CMPLX( w1(ih), w2(ih), KIND=dp )
-                          !
-                          ENDDO
-                        !
-                        jqs = iqs
-                        !
-                        ENDDO
-                        !
+                            !
+                         ENDDO
+                         !
+                         jqs = iqs
+                         !
+                      ENDDO
+                      !
                       DEALLOCATE( w1, w2 )
-                        !
-                      ENDIF
-                    !
-                    ENDDO
-                  !
-                 ENDDO
-
+                      !
+                   ENDIF
+                   !
+                ENDDO
+                !
+             ENDDO
+             !
           ENDIF
           !
           !   Back to reciprocal space 
@@ -513,8 +550,12 @@ CONTAINS
           !
        ENDDO
        !
+       !$acc exit data delete (dvrss)
+       !
 #if defined(__MPI)
+       !$acc host_data use_device(evc1_new)
        CALL mp_sum( evc1_new(:,:,1), inter_bgrp_comm )
+       !$acc end host_data
 #endif
        IF (dffts%has_task_groups) DEALLOCATE (tg_dvrss)
        !
@@ -525,7 +566,7 @@ CONTAINS
           !
        ENDIF
        !
-       CALL stop_clock('interaction')
+       CALL stop_clock_gpu('interaction')
        !
     ENDIF
     !
@@ -541,11 +582,7 @@ CONTAINS
     ! Compute sevc1_new = H*evc1
     !
 #if defined(__CUDA)
-    !$acc data copyin(evc1) copyout(sevc1_new)
-    !$acc host_data use_device(evc1, sevc1_new)
     CALL h_psi_gpu (npwx,ngk(1),nbnd,evc1(1,1,1),sevc1_new(1,1,1))
-    !$acc end host_data
-    !$acc end data
 #else
     CALL h_psi(npwx,ngk(1),nbnd,evc1(1,1,1),sevc1_new(1,1,1))
 #endif
@@ -562,11 +599,7 @@ CONTAINS
         ENDDO
     ELSE
 #if defined(__CUDA)
-       !$acc data copyin(evc1) copyout(spsi1)
-       !$acc host_data use_device(evc1, spsi1)
        CALL s_psi_acc (npwx,ngk(1),nbnd,evc1(1,1,1),spsi1)
-       !$acc end host_data
-       !$acc end data
 #else            
        CALL s_psi(npwx,ngk(1),nbnd,evc1(1,1,1),spsi1)
 #endif
@@ -576,11 +609,16 @@ CONTAINS
     !
     DO ibnd = 1,nbnd
        !
-       CALL zaxpy(ngk(1), CMPLX(-(et(ibnd,1)-scissor),0.0d0,DP), &
-                      & spsi1(:,ibnd), 1, sevc1_new(:,ibnd,1), 1)
+       coef = CMPLX(-(et(ibnd,1)-scissor),0.0d0)
+       !$acc host_data use_device(spsi1, sevc1_new) 
+       CALL zaxpy(ngk(1), coef, &
+             & spsi1(1,ibnd), 1, sevc1_new(1,ibnd,1), 1)
+       !$acc end host_data       
        !
     ENDDO
     !
+    !$acc end data 
+    !
     IF ( nkb > 0 .and. okvan ) DEALLOCATE(becp2)
     !
     RETURN
